
-- ====== Common =======  --

-- Desc: Получает ID текущего семестра
-- Note: uses by another functions
DROP FUNCTION IF EXISTS GetCurSemesterID;//
CREATE FUNCTION `GetCurSemesterID`( ) RETURNS int(11)
    NO SQL
BEGIN
	RETURN (SELECT general_settings.Val FROM `general_settings` WHERE general_settings.ID = 1 );
END //


-- Desc: Получает ID семестра
-- Note: uses by another functions
DROP FUNCTION IF EXISTS GetSemesterID;//
CREATE FUNCTION `GetSemesterID`( `Year` INT, `Num` INT ) RETURNS int(11)
    NO SQL
BEGIN
	RETURN (SELECT semesters.ID FROM `semesters` WHERE semesters.Year = Year AND semesters.Num = Num );
END //


-- Desc: Получает информацию о текущем семестре
DROP PROCEDURE IF EXISTS GetCurSemesterInfo;//
CREATE PROCEDURE `GetCurSemesterInfo`( )
    NO SQL
BEGIN
	SELECT 	semesters.ID, 
			semesters.Num, 
			semesters.Year 
	FROM `general_settings`
	INNER JOIN `semesters`  ON general_settings.Val = semesters.ID
	WHERE general_settings.ID = 1;
END //

DROP FUNCTION IF EXISTS SetHashKey;//
CREATE FUNCTION `SetHashKey`( `Value` VARCHAR(300) ) RETURNS int(11)
    NO SQL
BEGIN
	UPDATE `general_settings`
	SET general_settings.ValS = Value
	WHERE general_settings.ID = 2;
	RETURN 0;
END //

DROP FUNCTION IF EXISTS GetHashKey;//
CREATE FUNCTION `GetHashKey`() RETURNS varchar(300) CHARSET utf8
    NO SQL
BEGIN
	RETURN (	SELECT	general_settings.ValS
	        	FROM 	`general_settings`
	        	WHERE general_settings.ID = 2
	       );
END //


-- ====== Работа с аккаунтами ======


-- Desc: Количество аккаунтов с данным кодом активации
-- Note: uses by another functions
DROP FUNCTION IF EXISTS GetAccNumByCode;//
CREATE FUNCTION `GetAccNumByCode`( `Code` VARCHAR(40) CHARSET utf8 ) RETURNS int(11)
    NO SQL
BEGIN
	RETURN (SELECT COUNT(accounts.ID) FROM accounts WHERE accounts.ActivationCode = Code );
END //


-- Desc: Количество аккаунтов с данным логином
DROP FUNCTION IF EXISTS GetAccNumByLogin; //
CREATE FUNCTION `GetAccNumByLogin`(`Login` VARCHAR(50) CHARSET utf8) RETURNS int(11)
    NO SQL
BEGIN 
	RETURN ( SELECT COUNT(accounts.ID) FROM accounts WHERE accounts.Login = Login );
END //


-- Desc: Количество аккаунтов с данным e-mail
DROP FUNCTION IF EXISTS GetAccNumByMail; //
CREATE FUNCTION `GetAccNumByMail`(`Mail` VARCHAR(255) CHARSET utf8) RETURNS int(11)
    NO SQL
BEGIN 
	RETURN ( SELECT COUNT(accounts.ID) FROM accounts WHERE accounts.`E-Mail` = Mail );
END //


-- Desc: * для аккаунта с данным логином и паролем
DROP PROCEDURE IF EXISTS GetAccInfoByID; //
CREATE PROCEDURE `GetAccInfoByID`(IN `UserID` VARCHAR(50) CHARSET utf8 )
    NO SQL
BEGIN 
	SELECT 	accounts.ID, 
			accounts.`Login`, 
			accounts.`E-Mail`, 
			accounts.Type, 
			( accounts.ActivationCode IS NULL) AS 'isActivated'
	FROM accounts
	WHERE accounts.`ID` = UserID;
END //






-- Desc: - UserID => FirstName, SecondName, LastName, StudyGroup (вся информация, в т.ч. и специализация (ПМ, ИТ, etc.)) (в зав. от AccountType)
-- Note: 	1) если аккаунт существует, то всегда можно получить UserID, AccountType, LastName, FirstName
--				иначе UserID = NULL (как и все остальные)
--			2) Degree у преподавателя и LastName могут быть NULL 
DROP PROCEDURE IF EXISTS GetPersonalInfoByID; //
CREATE PROCEDURE `GetPersonalInfoByID`(IN `UserID` INT )
    NO SQL
BEGIN 
	DECLARE varType INT;
	SELECT accounts.Type FROM accounts WHERE accounts.ID = UserID LIMIT 1 INTO varType;
	IF varType = 1 THEN 
		
		SELECT 	UserID 					AS 'UserID', 
				accounts.Type 			AS 'AccountType', 
				students.ID 			AS 'StudentID', 
				students.LastName, 
				students.FirstName, 
				students.SecondName, 
				study_groups.ID 		AS 'StudyGroupID', 
				study_groups.Grade 		AS 'Grade', 
				study_groups.GroupNum 	AS 'Group', 
				study_groups.Name 		AS 'StudyGroupName',
				specializations.ID 		AS 'SpecID', 
				specializations.Name	AS 'SpecName', 
				specializations.Abbr 	AS 'SpecAbbr', 
				faculties.ID 			AS 'FacultyID', 
				faculties.Name 			AS 'FacultyName', 
				faculties.Abbr 			AS 'FacultyAbbreviation',
				accounts.isEnabled,
				( accounts.ActivationCode IS NULL) AS 'isActivated',
				accounts.UserAgent
		FROM accounts
		INNER JOIN students ON UserID = AccountID
		INNER JOIN study_groups ON students.StudyGroupID = study_groups.ID
		INNER JOIN faculties ON study_groups.FacultyID = faculties.ID
		INNER JOIN specializations ON study_groups.SpecializationID = specializations.ID
		WHERE accounts.ID = UserID;

	ELSE
		SELECT 	UserID 					AS 'UserID', 
				accounts.Type 			AS 'AccountType', 
				teachers.ID 			AS 'TeacherID', 
				teachers.LastName, 
				teachers.FirstName, 
				teachers.SecondName, 
				teachers.Degree, 
				faculties.ID 			AS 'FacultyID', 
				faculties.Name 			AS 'FacultyName', 
				faculties.Abbr 			AS 'FacultyAbbreviation',
				accounts.isEnabled,
				( accounts.ActivationCode IS NULL) AS 'isActivated',
				accounts.UserAgent
		FROM `accounts`
		INNER JOIN `teachers` ON UserID = teachers.AccountID
		INNER JOIN `faculties` ON teachers.FacultyID = faculties.ID
		WHERE accounts.ID = UserID;

	END IF;

END //


-- Desc: Активация аккаунта (внесение данных изменений в поля Login, E-Mail, Password + обнуление activation_code и снятие флага isActivated). Кроме того, также установка AccountType в зависимости от связанной с ним информации.
DROP FUNCTION IF EXISTS ActivateAccount; //
CREATE FUNCTION `ActivateAccount`(`Code` VARCHAR(40) CHARSET utf8, `Login` VARCHAR(50) CHARSET utf8, `Mail` VARCHAR(32) CHARSET utf8, `Password` VARCHAR(255) CHARSET utf8) RETURNS int(11)
    NO SQL
BEGIN
	DECLARE checker INT;
	DECLARE UserID INT;
	SELECT COUNT(accounts.ID) INTO checker FROM accounts WHERE accounts.ActivationCode = Code;
	IF ( Password IS NULL ) THEN
 		return -2;
	END IF;  

	IF checker = 1 THEN
	    	SELECT accounts.ID 
	    	INTO UserID 
	    	FROM `accounts`
	    	WHERE accounts.ActivationCode = Code; 

        	UPDATE `accounts` 
    		SET 	accounts.Login = Login , 
    				accounts.Password = Password , 
    				accounts.`E-Mail` = Mail , 
    				accounts.ActivationCode = NULL
    		WHERE accounts.ID = UserID ;
    		
    		RETURN UserID;
	ELSE 
		RETURN -1;
	END IF;
END //



-- Desc: - Изменение пароля для данной учетной записи
DROP FUNCTION IF EXISTS ChangePassword; //
CREATE FUNCTION `ChangePassword`(`UserID` INT,  `Password` VARCHAR(255) CHARSET utf8) RETURNS int(11)
    NO SQL
BEGIN
	DECLARE checker INT;
	SET checker = ( SELECT COUNT(accounts.ID) FROM accounts WHERE accounts.ID = UserID );

	IF checker = 1 THEN  
    	UPDATE accounts 
    	SET accounts.Password = Password
    	WHERE ID = UserID;
    	
    	RETURN 0;
    ELSE  
      	RETURN -1; 
	END IF; 
END //


-- Desc: - Изменение логина для данной учетной записи
DROP FUNCTION IF EXISTS ChangeLogin; //
CREATE FUNCTION `ChangeLogin`(`UserID` INT,  `Login` VARCHAR(50) CHARSET utf8) RETURNS int(11)
    NO SQL
BEGIN
	DECLARE checker INT;

	SET checker = ( SELECT COUNT(accounts.ID) FROM accounts WHERE accounts.ID = UserID );
	IF checker != 1 THEN
		RETURN -1;
	END IF;

	SET checker = -1;
	SELECT accounts.ID 
	INTO checker
	FROM accounts
	WHERE accounts.Login = Login
	LIMIT 1;
	IF checker > 0 THEN 
		RETURN -1;
	END IF;

	UPDATE accounts 
	SET accounts.Login = Login
	WHERE ID = UserID;
	
	RETURN 0;
END //


-- Desc: - Изменение e-mail для данной учетной записи
DROP FUNCTION IF EXISTS ChangeMail; //
CREATE FUNCTION `ChangeMail`(`UserID` INT,  `mailN` VARCHAR(32) CHARSET utf8) RETURNS int(11)
    NO SQL
BEGIN
	DECLARE checker INT;
	SET checker = ( SELECT COUNT(accounts.ID) FROM accounts WHERE accounts.ID = UserID );
	IF checker != 1 THEN
		RETURN -1;
	END IF;

	SET checker = -1;
	SELECT accounts.ID 
	INTO checker
	FROM accounts
	WHERE accounts.`E-Mail` = mailN
	LIMIT 1;
	IF checker > 0 THEN 
		RETURN -1;
	END IF;


	UPDATE accounts 
	SET accounts.`E-Mail` = mailN
	WHERE ID = UserID;
	
	RETURN 0;
	
END //







-- Desc: Добавление нового студента
DROP FUNCTION IF EXISTS CreateStudent; //
CREATE FUNCTION `CreateStudent`(	`Last` 			VARCHAR(30) CHARSET utf8, 
									`First` 		VARCHAR(30) CHARSET utf8, 
									`Second` 		VARCHAR(30) CHARSET utf8, 
									`Grade` 		INT, 
									`GroupN` 		INT, 
									`FacultyID` 	INT, 
									`Code` 			VARCHAR(40) CHARSET utf8 
								) RETURNS int(11)
    NO SQL
BEGIN 
	DECLARE curID, StudyGroupID INT;
	SET StudyGroupID = -1;
	
	SELECT study_groups.ID 
	INTO StudyGroupID 
	FROM `study_groups` 
	WHERE 	(study_groups.Grade = Grade ) 	AND 
			( study_groups.GroupNum = GroupN ) AND 
			( study_groups.FacultyID = FacultyID ) 
	LIMIT 1;

	IF ( (StudyGroupID > -1) AND ( Code IS NOT NULL ) ) THEN

		INSERT INTO `accounts` ( accounts.Login , accounts.Password , accounts.`E-Mail`, accounts.Type, accounts.ActivationCode )
		VALUES 				   ( NULL, NULL, NULL, 'student', Code);
		
		SELECT accounts.ID 
		INTO curID
		FROM `accounts`
		WHERE accounts.Code = Code;
		
		INSERT INTO `students` (students.StudyGroupID, students.AccountID, students.LastName, students.FirstName, students.SecondName)
		VALUES 				   (StudyGroupID, curID, Last, First, Second);

		RETURN 0;



	END IF;
	RETURN -1;
END //



DROP FUNCTION IF EXISTS CreateTeacher; //
CREATE FUNCTION `CreateTeacher`(	`Last` 		VARCHAR(30) CHARSET utf8, 
									`First` 	VARCHAR(30) CHARSET utf8, 
									`Second` 	VARCHAR(30) CHARSET utf8, 
									`Degree` 	VARCHAR(50) CHARSET utf8, 
									`FacultyID` INT, 
									`Code` 		VARCHAR(40) CHARSET utf8 
								) RETURNS int(11)
    NO SQL
BEGIN 
	DECLARE curID, StudyGroupID INT;

	IF Code IS NOT NULL THEN

		INSERT INTO `accounts` ( accounts.Login , accounts.Password , accounts.`E-Mail`, accounts.Type, accounts.ActivationCode )
		VALUES 				   ( NULL, NULL, NULL, 'teacher', Code);
		
		SELECT accounts.ID INTO curID
		FROM `accounts`
		WHERE accounts.Code = Code;
		
		INSERT INTO `teachers` (	teachers.AccountID, 
									teachers.LastName, 
									teachers.FirstName, 
									teachers.SecondName, 
									teachers.Degree, 
									teachers.FacultyID)
		VALUES 				   (curID, Last, First, Second, Degree, FacultyID);
		RETURN 0;

	END IF;
	RETURN -1;
END //



DROP FUNCTION IF EXISTS SignIn; //
CREATE FUNCTION `SignIn`(	`Login` 	VARCHAR(255) CHARSET utf8, 
							`Password` 	VARCHAR(50) CHARSET utf8
						) RETURNS int(11)
    NO SQL
BEGIN 
	DECLARE curID INT;
	SELECT accounts.ID
	INTO curID
	FROM `accounts`
	WHERE 	(accounts.Login = Login OR accounts.`E-Mail` = Login) AND
			accounts.Password = Password
	LIMIT 1;

	IF curID > 0 THEN 
		RETURN curID;
	ELSE 
		RETURN -1;
	END IF;

END //




-- ======Студенты=========



DROP FUNCTION IF EXISTS GetRateForDisc; //
CREATE FUNCTION `GetRateForDisc`(`StudentID` INT,  `DisciplineID` INT) RETURNS int(11)
    NO SQL
BEGIN
	RETURN 	(	SELECT SUM(rating_table.Rate) 
				FROM `modules`
				CROSS JOIN `submodules` ON submodules.ModuleID = modules.ID
				CROSS JOIN `rating_table` ON 	rating_table.StudentID = StudentID AND 
												rating_table.SubmoduleID = submodules.ID
				WHERE modules.DisciplineID = DisciplineID 
			);
END //


DROP FUNCTION IF EXISTS GetMaxRateForDisc; //
CREATE FUNCTION `GetMaxRateForDisc`(`StudentID` INT,  `DisciplineID` INT) RETURNS int(11)
    NO SQL
BEGIN
	RETURN 	(	SELECT SUM(submodules.MaxRate) 
				FROM `modules`
				CROSS JOIN `submodules` ON submodules.ModuleID = modules.ID
				CROSS JOIN `rating_table` ON 	rating_table.StudentID = StudentID AND 
												rating_table.SubmoduleID = submodules.ID
				WHERE modules.DisciplineID = DisciplineID 
			);
END //




-- Desc: Все курсы, на которые подписан данный студент + преподаватель курса + тип контроля (экзамен/зачет) + баллы + максимальное количество баллов на данный момент (IN: StudentID).
DROP PROCEDURE IF EXISTS GetDisciplinesForStudent; //
CREATE PROCEDURE `GetDisciplinesForStudent`(IN `StudentID` INT)
    NO SQL
BEGIN  
    DECLARE groupID, semID INT;  
    SELECT students.StudyGroupID INTO groupID FROM `students` WHERE students.ID = StudentID LIMIT 1;  						 
	SET semID = GetCurSemesterID();
	
	SELECT 	DISTINCT	disciplines.ID AS 'DisciplineID', 
						subjects.ID AS 'SubjectID', 
						subjects.Name AS 'SubjectName', 
						disciplines.ExamType,
						teachers.LastName AS 'TeacherLast', 
						teachers.FirstName AS 'TeacherFirst',
						teachers.SecondName AS 'TeacherSecond', 
						teachers.Degree AS 'TeacherDegree',
						GetRateForDisc(StudentID, `disciplines`.`ID` ) AS 'Rate', 
						GetMaxRateForDisc(StudentID, `disciplines`.`ID` ) AS 'MaxCurrentRate' 
	FROM `disciplines`
	INNER JOIN `subjects` 					ON 	disciplines.SubjectID = subjects.ID 
	INNER JOIN `teachers` 					ON 	disciplines.AuthorID = teachers.ID
	CROSS JOIN `disciplines_teachers` 		ON 	disciplines_teachers.DisciplineID = disciplines.ID
	CROSS JOIN `lessons`					ON 	lessons.DisciplineTeacherID = disciplines_teachers.ID
	INNER JOIN `attending_groups`			ON 	attending_groups.ID = lessons.AttendingGroupID AND 
												attending_groups.SemesterID = semID
	LEFT JOIN 	`attending_groups_bases` 	ON 	attending_groups_bases.AttendingGroupID = attending_groups.ID AND
												attending_groups_bases.StudyGroupID = groupID
	LEFT JOIN 	`attending_groups_pool`		ON 	attending_groups_pool.AttendingGroupID = attending_groups.ID AND
												attending_groups_pool.StudentID = StudentID
	WHERE 	disciplines.SemesterID = semID AND
			attending_groups.SemesterID = semID AND
			(	attending_groups_pool.Type = 'attach'	OR
				(	(attending_groups_pool.Type is NULL OR attending_groups_pool.Type != 'detach') AND 
					attending_groups_bases.StudyGroupID IS NOT NULL AND 
					attending_groups_bases.StudyGroupID = groupID 
				)
			)
	ORDER BY disciplines.ExamType ASC, Rate DESC;   
END //



-- Decs: Полная информация по модулям/подмодулям для данного студента по данному курсу (заголовки модулей, максимальное количество баллов за подмодуль)	
DROP PROCEDURE IF EXISTS GetSubjectMapForStudent; //
CREATE PROCEDURE `GetSubjectMapForStudent`(IN `StudentID` INT, IN `SubjectID` INT)
BEGIN  
    DECLARE groupID, semID INT;  
    SELECT students.StudyGroupID INTO groupID FROM `students` WHERE students.ID = StudentID LIMIT 1;  						 
	SET semID = GetCurSemesterID();

	SELECT 	modules.ID AS 'ModuleID', 
			modules.Name AS 'ModuleName', 
			submodules.ID AS 'SubmoduleID', 
			submodules.Name AS 'SubModuleName', 
			submodules.Description AS 'SubmoduleDescription', 
			submodules.MaxRate,
			submodules.Type 		AS 'SubmoduleControl',
			rating_table.Rate, 
			rating_table.Date
	FROM `submodules`
	INNER JOIN `modules` ON submodules.ModuleID = modules.ID
	LEFT JOIN `rating_table` ON submodules.ID = rating_table.SubModuleID AND rating_table.StudentID = StudentID 
	WHERE modules.DisciplineID IN 
		(		SELECT 	DISTINCT	disciplines.ID 
				FROM `disciplines`
				CROSS JOIN `disciplines_teachers` 		ON 	disciplines_teachers.DisciplineID = disciplines.ID
				CROSS JOIN `lessons`					ON 	lessons.DisciplineTeacherID = disciplines_teachers.ID
				INNER JOIN `attending_groups`			ON 	attending_groups.ID = lessons.AttendingGroupID AND 
															attending_groups.SemesterID = semID
				LEFT JOIN 	`attending_groups_bases` 	ON 	attending_groups_bases.AttendingGroupID = attending_groups.ID AND
															attending_groups_bases.StudyGroupID = groupID
				LEFT JOIN 	`attending_groups_pool`		ON 	attending_groups_pool.AttendingGroupID = attending_groups.ID AND
															attending_groups_pool.StudentID = StudentID
				WHERE 	disciplines.SubjectID = SubjectID 	AND
						disciplines.SemesterID = semID 		AND
						attending_groups.SemesterID = semID AND
						(	attending_groups_pool.Type = 'attach'	OR
							(	(attending_groups_pool.Type is NULL OR attending_groups_pool.Type != 'detach') AND 
								attending_groups_bases.StudyGroupID IS NOT NULL AND 
								attending_groups_bases.StudyGroupID = groupID 
							)
						)
		)
	ORDER BY modules.OrderNum ASC, submodules.OrderNum ASC;
END //


-- ======= Преподаватели =======



DROP PROCEDURE IF EXISTS GetDisciplineInfoByID; //
CREATE PROCEDURE `GetDisciplineInfoByID`(IN `DiscID` INT)
    NO SQL
BEGIN  

	SELECT 	disciplines.AuthorID,
			disciplines.Grade 			AS 'DisciplineGrade',
			disciplines.ExamType,
			disciplines.Type 			AS 'LessonType',
			disciplines.LectionCount,
			disciplines.PracticeCount,
			disciplines.SemesterID,
			subjects.ID 				AS 'SubjectID',
			subjects.Name 				AS 'SubjectName',
			subjects.Abbr 				AS 'SubjectAbbr'


	FROM `disciplines`
	INNER JOIN `subjects` ON subjects.ID = disciplines.SubjectID
	WHERE disciplines.ID = DiscID;
END //


DROP PROCEDURE IF EXISTS GetTeachersForDiscipline; //
CREATE PROCEDURE `GetTeachersForDiscipline`(IN `DiscID` INT)
    NO SQL
BEGIN  

		SELECT 	teachers.ID 			AS 'TeacherID', 
				teachers.LastName, 
				teachers.FirstName, 
				teachers.SecondName, 
				teachers.Degree, 
				faculties.ID 			AS 'FacultyID', 
				faculties.Name 			AS 'FacultyName', 
				faculties.Abbr 			AS 'FacultyAbbreviation'
		FROM `disciplines_teachers`
		INNER JOIN `teachers` ON teachers.ID = disciplines_teachers.TeacherID
		INNER JOIN `faculties` ON teachers.FacultyID = faculties.ID
		WHERE disciplines_teachers.DisciplineID = DiscID;
END //





-- Desc: Вывод всех курсов (ID курса, название курса, базовая студ.группа-слушатель), которые ведет преподаватель
DROP PROCEDURE IF EXISTS GetDisciplinesForTeacher; //
CREATE PROCEDURE `GetDisciplinesForTeacher`(IN `TeacherID` INT)
    NO SQL
BEGIN  
    DECLARE groupID, semID INT;  						 
	SET semID = GetCurSemesterID();

	SELECT DISTINCT disciplines.ID 			AS 'DisciplineID',
                    disciplines.ExamType 	AS 'DisciplineType',
                    subjects.ID 			AS 'SubjectID',
                    subjects.Name 			AS 'SubjectName',
                    disciplines_teachers.ID AS 'DisciplineTeacherID',
                    study_groups.ID 		AS 'StudyGroupID', 
                    study_groups.GroupNum 	AS 'StudyGroupNum', 
                    study_groups.Grade 		AS 'StudyGroupGrade',
                    study_groups.Name 		AS 'StudyGroupName' 
    FROM `disciplines_teachers`
    INNER JOIN `disciplines` 			ON disciplines.ID = disciplines_teachers.DisciplineID
    INNER JOIN `subjects`				ON subjects.ID = disciplines.SubjectID
    CROSS JOIN `lessons` 				ON lessons.DisciplineTeacherID = disciplines_teachers.ID
    INNER JOIN `attending_groups` 		ON attending_groups.ID = lessons.AttendingGroupID
    CROSS JOIN `attending_groups_bases` ON attending_groups_bases.AttendingGroupID = attending_groups.ID
    INNER JOIN `study_groups` 			ON study_groups.ID = attending_groups_bases.StudyGroupID
    WHERE 	disciplines_teachers.TeacherID = TeacherID AND 
    		disciplines.SemesterID = semID
    ORDER BY 	disciplines.Grade 		ASC, 
    			subjects.Name 			ASC,
    			study_groups.GroupNum 	ASC;
END //



-- Desc: Вывод всех УКД (ID УКД [я же надеюсь, что у нас УКД хранятся в виде конкретных осязаемых сущностей?], которые неявно прикреплены к преподавателю
DROP PROCEDURE IF EXISTS GetMapsForTeacher; //
CREATE PROCEDURE `GetMapsForTeacher`(IN `TeacherID` INT)
    NO SQL
BEGIN  
    DECLARE semID INT;  					 
	SET semID = GetCurSemesterID();

	SELECT 	subjects.ID 			AS 'SubjectID',
			subjects.Name 			AS 'SubjectName',	
			modules.ID 				AS 'ModuleID', 
			modules.Name 			AS 'ModuleName', 
			submodules.ID 			AS 'SubmoduleID', 
			submodules.Name 		AS 'SubModuleName', 
			submodules.Description 	AS 'SubmoduleDescription', 
			submodules.MaxRate 		AS 'MaxRate',
			submodules.Type 		AS 'SubmoduleControl'
	FROM `disciplines_teachers`
	INNER JOIN `disciplines` ON disciplines_teachers.DisciplineID = disciplines.ID
	INNER JOIN `subjects` ON disciplines.SubjectID = subjects.ID
	CROSS JOIN `modules` ON modules.DisciplineID = disciplines.ID
	CROSS JOIN `submodules` ON submodules.ModuleID = modules.ID
	WHERE disciplines_teachers.TeacherID = TeacherID AND disciplines.SemesterID = semID
	ORDER BY 	modules.OrderNum ASC, 
				submodules.OrderNum ASC;
END //








DROP FUNCTION IF EXISTS InternalOccurrenceChecker ; //
CREATE FUNCTION `InternalOccurrenceChecker`	(	`StudentID` INT, 
												`AG_ID` 	INT ) 
												RETURNS int(11)
    NO SQL
BEGIN

	DECLARE mID INT;
	SET mID = 0; 
	SELECT students.ID 
	INTO mID
	FROM `students`
	INNER JOIN `study_groups` ON students.StudyGroupID = study_groups.ID
	LEFT JOIN `attending_groups_bases` ON attending_groups_bases.AttendingGroupID = AG_ID AND attending_groups_bases.StudyGroupID = study_groups.ID
	LEFT JOIN `attending_groups_pool` ON attending_groups_pool.AttendingGroupID = AG_ID AND attending_groups_pool.StudentID = StudentID
	WHERE 	students.ID = StudentID AND
			(( attending_groups_pool.Type IS NOT NULL AND attending_groups_pool.Type = 'attach')	OR
			( (attending_groups_pool.Type IS NULL OR attending_groups_pool.Type != 'detach') AND attending_groups_bases.ID is NOT NULL)
			)
	LIMIT 1;

	RETURN (mID>0);

END //



-- Вывод всех студентов, которые слушают данный курс данного преподавателя
DROP PROCEDURE IF EXISTS GetStudentsForDiscipline; //
CREATE PROCEDURE `GetStudentsForDiscipline`(IN `TeacherID` INT,
											IN `DisciplineID` INT )
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT disciplines_teachers.ID 
	INTO checker 
	FROM `disciplines_teachers`
	WHERE 	disciplines_teachers.DisciplineID = DisciplineID AND 
			disciplines_teachers.TeacherID = TeacherID
	LIMIT 1;

	IF checker > 0 THEN
		SELECT DISTINCT 	students.ID AS 'StudentID',
							students.LastName,
							students.FirstName,
							students.SecondName,
							study_groups.Grade,
							study_groups.GroupNum,
							(attending_groups_pool.StudentID IS NOT NULL) AS 'isAttached'

	    FROM `disciplines_teachers`
	    INNER JOIN `disciplines` 		ON disciplines.ID = disciplines_teachers.DisciplineID
	    CROSS JOIN `lessons` 			ON lessons.DisciplineTeacherID = disciplines_teachers.ID
	    INNER JOIN `attending_groups` 	ON attending_groups.ID = lessons.AttendingGroupID
	    CROSS JOIN `students` 			ON InternalOccurrenceChecker(students.ID, attending_groups.ID) = TRUE
    	LEFT JOIN  `attending_groups_pool` ON attending_groups_pool.AttendingGroupID = attending_groups.ID AND attending_groups_pool.StudentID = students.ID
	    INNER JOIN `study_groups` 		ON students.StudyGroupID = study_groups.ID
	    WHERE 	disciplines_teachers.TeacherID = TeacherID AND 
	    		disciplines_teachers.DisciplineID = DisciplineID
		ORDER BY 	study_groups.Grade 		ASC, 
					study_groups.GroupNum 	ASC,
					students.LastName 		ASC,
					students.FirstName 		ASC,
					students.SecondName 	ASC;
	ELSE 
		SELECT 	NULL AS 'StudentID',
				NULL AS 'LastName',
				NULL AS 'FirstName',
				NULL AS 'SecondName',
				NULL AS 'Grade',
				NULL AS 'Group'
		LIMIT 0;

	END IF;
END //





-- Вывод всех модулей/подмодулей для данного курса данного преподавателя
DROP PROCEDURE IF EXISTS GetMapForDiscipline; //
CREATE PROCEDURE `GetMapForDiscipline`(	IN `TeacherID` 		INT,
										IN `DisciplineID` 	INT )
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT disciplines_teachers.ID 
	INTO checker 
	FROM `disciplines_teachers`
	WHERE 	disciplines_teachers.DisciplineID = DisciplineID AND 
			disciplines_teachers.TeacherID = TeacherID
	LIMIT 1;

	IF checker > 0 THEN
		SELECT 	subjects.ID 			AS 'SubjectID',
				subjects.Name 			AS 'SubjectName',	
				modules.ID 				AS 'ModuleID', 
				modules.Name 			AS 'ModuleName', 
				submodules.ID 			AS 'SubmoduleID', 
				submodules.Name 		AS 'SubModuleName', 
				submodules.Description 	AS 'SubmoduleDescription', 
				submodules.MaxRate,
				submodules.Type 		AS 'SubmoduleControl'
		FROM `disciplines`
		INNER JOIN `subjects` 	ON disciplines.SubjectID = subjects.ID
		CROSS JOIN `modules` 	ON modules.DisciplineID = disciplines.ID
		CROSS JOIN `submodules` ON submodules.ModuleID = modules.ID
		WHERE disciplines.ID = DisciplineID
		ORDER BY modules.OrderNum ASC, 
				 submodules.OrderNum ASC;
 	ELSE 
		SELECT 	NULL AS 'SubjectID',
				NULL AS 'SubjectName',	
				NULL AS 'ModuleID', 
				NULL AS 'ModuleName', 
				NULL AS 'SubmoduleID', 
				NULL AS 'SubModuleName', 
				NULL AS 'SubmoduleDescription', 
				NULL AS 'MaxRate'
		LIMIT 0;
	END IF;
END //



-- Добавление баллов к данному студенту
DROP FUNCTION IF EXISTS SetStudentRate; //
CREATE FUNCTION `SetStudentRate`(	`TeacherID` 	INT,
									`StudentID` 	INT,
									`SubmoduleID` 	INT,
									`Rate` 			INT ) 
									RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker, discID, groupID, semID INT; 
    SET discID = -1;
    SET groupID = -1; 					 
	
    SELECT disciplines.ID 
    INTO discID
	FROM `submodules`
	INNER JOIN `modules` 		ON submodules.ModuleID = modules.ID
	INNER JOIN `disciplines` 	ON disciplines.ID = modules.DisciplineID 
	WHERE submodules.ID = SubmoduleID
	LIMIT 1;

	SELECT disciplines.ID 
	INTO checker 
	FROM `disciplines`
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID AND disciplines_teachers.TeacherID = TeacherID  
	WHERE disciplines.ID = discID
	LIMIT 1;

	IF checker > 0 THEN

		SELECT COUNT(rating_table.ID) 
		INTO checker
		FROM `rating_table`
		WHERE 	rating_table.SubmoduleID = SubmoduleID AND 
				rating_table.TeacherID = TeacherID AND 
				rating_table.StudentID = StudentID;

		IF checker = 0 THEN

			SELECT students.StudyGroupID INTO groupID FROM `students` WHERE students.ID = StudentID LIMIT 1; 
			SET semID = GetCurSemesterID(); 
			
			SELECT 	disciplines.ID INTO checker
			FROM `disciplines`
			CROSS JOIN `disciplines_teachers` 		ON 	disciplines_teachers.DisciplineID = disciplines.ID
			CROSS JOIN `lessons`					ON 	lessons.DisciplineTeacherID = disciplines_teachers.ID
			INNER JOIN `attending_groups`			ON 	attending_groups.ID = lessons.AttendingGroupID AND 
														attending_groups.SemesterID = semID
			LEFT JOIN 	`attending_groups_bases` 	ON 	attending_groups_bases.AttendingGroupID = attending_groups.ID AND
														attending_groups_bases.StudyGroupID = groupID
			LEFT JOIN 	`attending_groups_pool`		ON 	attending_groups_pool.AttendingGroupID = attending_groups.ID AND
														attending_groups_pool.StudentID = StudentID
			WHERE 	disciplines.SubjectID = SubjectID 	AND
					disciplines.SemesterID = semID 		AND
					attending_groups.SemesterID = semID AND
					(	attending_groups_pool.Type = 'attach'	OR
						(	(attending_groups_pool.Type is NULL OR attending_groups_pool.Type != 'detach') AND 
							attending_groups_bases.StudyGroupID IS NOT NULL AND 
							attending_groups_bases.StudyGroupID = groupID 
						)
					)
			LIMIT 1;

			IF checker > 0 THEN
				INSERT INTO `rating_table` ( rating_table.StudentID, rating_table.TeacherID, rating_table.SubmoduleID, rating_table.Rate, rating_table.Date )
				VALUES ( StudentID, TeacherID, SubmoduleID, Rate, CURDATE() );
				RETURN 0;
			END IF;
			RETURN -3;
		END IF;
		RETURN checker;
	END IF;
	RETURN -1;
END //


-- Изменение баллов для данного студента
DROP FUNCTION IF EXISTS ChangeStudentRate; //
CREATE FUNCTION `ChangeStudentRate`	(	`TeacherID` 	INT,
										`StudentID` 	INT,
										`SubmoduleID` 	INT,
										`Rate` 			INT ) 
										RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker, discID, OldVal, accID, groupID, rateID, semID INT;
    SET discID = -1;
    SET groupID = -1;  					 
	
    SELECT disciplines.ID 
    INTO discID
	FROM `submodules`
	INNER JOIN `modules` 	 	ON submodules.ModuleID = modules.ID
	INNER JOIN `disciplines` 	ON disciplines.ID = modules.DisciplineID 
	WHERE submodules.ID = SubmoduleID
	LIMIT 1;

	SELECT disciplines.ID 
	INTO checker 
	FROM `disciplines`
	INNER JOIN `disciplines_teachers` ON 	disciplines_teachers.DisciplineID = disciplines.ID AND 
											disciplines_teachers.TeacherID = TeacherID  
	WHERE disciplines.ID = discID
	LIMIT 1;

	
	IF checker > 0 THEN
		SELECT rating_table.ID 
		INTO rateID
		FROM `rating_table`
		WHERE 	rating_table.SubmoduleID = SubmoduleID AND 
				rating_table.TeacherID = TeacherID AND 
				rating_table.StudentID = StudentID
		LIMIT 1;

		IF checker > 0 THEN
			SELECT students.StudyGroupID INTO groupID FROM `students` WHERE students.ID = StudentID LIMIT 1; 
			SET semID = GetCurSemesterID(); 
			
			SELECT 	disciplines.ID INTO checker
			FROM `disciplines`
			CROSS JOIN `disciplines_teachers` 		ON 	disciplines_teachers.DisciplineID = disciplines.ID
			CROSS JOIN `lessons`					ON 	lessons.DisciplineTeacherID = disciplines_teachers.ID
			INNER JOIN `attending_groups`			ON 	attending_groups.ID = lessons.AttendingGroupID AND 
														attending_groups.SemesterID = semID
			LEFT JOIN 	`attending_groups_bases` 	ON 	attending_groups_bases.AttendingGroupID = attending_groups.ID AND
														attending_groups_bases.StudyGroupID = groupID
			LEFT JOIN 	`attending_groups_pool`		ON 	attending_groups_pool.AttendingGroupID = attending_groups.ID AND
														attending_groups_pool.StudentID = StudentID
			WHERE 	disciplines.SubjectID = SubjectID 	AND
					disciplines.SemesterID = semID 		AND
					attending_groups.SemesterID = semID AND
					(	attending_groups_pool.Type = 'attach'	OR
						(	(attending_groups_pool.Type is NULL OR attending_groups_pool.Type != 'detach') AND 
							attending_groups_bases.StudyGroupID IS NOT NULL AND 
							attending_groups_bases.StudyGroupID = groupID 
						)
					)
			LIMIT 1;

			IF checker > 0 THEN	
				SELECT rating_table.Rate 
				INTO OldVal
				FROM `rating_table`
				WHERE rating_table.ID = rateID
				LIMIT 1;


				SELECT accounts.ID 
				INTO accID
				FROM `teachers`
				INNER JOIN `accounts` ON teachers.AccountID = accounts.ID
				WHERE teachers.ID = TeacherID
				LIMIT 1; 

				INSERT INTO `logs` ( logs.RatingTableID, logs.AccountID, logs.OldValue, logs.NewValue, logs.Date )
				VALUES ( rateID, accID, OldVal, Rate, NOW() );
			

				UPDATE `rating_table`
				SET rating_table.Rate = Rate
				WHERE rating_table.ID = rateID;

				RETURN 0;


			END IF;
		END IF;
	END IF;
	RETURN -1;
END //



-- ====== УКД ============= --

DROP FUNCTION IF EXISTS AddDiscipline; //
CREATE FUNCTION `AddDiscipline`(	`TeacherID` 	INT,
									`Grade`			INT,
									`SubjectID`		INT,
									`ExamType`		VARCHAR(30),
									`LectionCount`  INT,
									`PracticeCount` INT			
								 ) 
								RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE semID, checker INT; 
    SET semID = GetCurSemesterID();   
	

	INSERT INTO `disciplines` 	(	disciplines.AuthorID, 
									disciplines.Grade, 
									disciplines.SubjectID, 
									disciplines.ExamType, 
									disciplines.LectionCount, 
									disciplines.PracticeCount )
	VALUES 					( TeacherID, Grade, SubjectID, ExamType, LectionCount, PracticeCount );
	
	-- RETURN 	(	SELECT disciplines.ID
	-- 			FROM   `disciplines`
	-- 			WHERE 	disciplines.AuthorID  	= TeacherID 	AND
	-- 					disciplines.SubjectID 	= SubjectID		AND
	-- 					disciplines.Grade 		= Grade
	-- 			ORDER BY disciplines.ID DESC
	-- 			LIMIT 1
	-- 		);
	RETURN ( LAST_INSERT_ID() );
	
END //



DROP FUNCTION IF EXISTS ChangeDiscipline; //
CREATE FUNCTION `ChangeDiscipline`(	`TeacherID` 	INT,
									`DisciplineID`	INT,
									`ExamType`		VARCHAR(30),
									`LectionCount`  INT,
									`PracticeCount` INT			
								 ) 
								RETURNS int(11)
    NO SQL
BEGIN 
    DECLARE checker INT;  					 
	
	SELECT disciplines_teachers.ID
	INTO checker 
	FROM `disciplines_teachers`
	WHERE disciplines_teachers.DisciplineID = DisciplineID AND disciplines_teachers.TeacherID 	= TeacherID
	LIMIT 1;

	IF checker IS NULL OR checker <=0 THEN 
		RETURN -1;
	END IF;


	UPDATE `disciplines`
	SET 	disciplines.ExamType = ExamType,
			disciplines.LectionCount = LectionCount,
			disciplines.PracticeCount = PracticeCount
	WHERE disciplines.ID = DisciplineID;

	RETURN 0;

END //



-- Внесение изменений в подмодуль (MaxRate, TItle, Description, Order и т.д.), который неявно прикреплен к преподавателю
DROP FUNCTION IF EXISTS ChangeSubmodule; //
CREATE FUNCTION `ChangeSubmodule`(	`TeacherID` 	INT, 
									`SubmoduleID` 	INT, 
									`MaxRate` 		INT, 
									`OrderNum` 		INT, 
									`Name` 			VARCHAR(50), 
									`Description` 	VARCHAR(200),
									`ControlType`	VARCHAR(30) 
								) RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT modules.ID 
	INTO checker 
	FROM `submodules`
	INNER JOIN `modules` 		ON 	submodules.ModuleID 	= modules.ID
	INNER JOIN `disciplines` 	ON 	disciplines.ID 			= modules.DisciplineID
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID AND disciplines_teachers.TeacherID 	= TeacherID
	WHERE 	submodules.ID = SubmoduleID 
	LIMIT 1;
	IF checker IS NULL OR checker <= 0 THEN
		RETURN -1;
	END IF;

	SET checker = -1;
	SELECT submodules.ID
	INTO checker
	FROM `submodules`
	WHERE submodules.ModuleID = checker AND submodules.OrderNum = OrderNum
	LIMIT 1;
	IF checker > 0 THEN
		RETURN -1;
	END IF; 

	UPDATE `submodules` 
	SET submodules.MaxRate 		= MaxRate, 
		submodules.OrderNum 	= OrderNum, 
		submodules.Name 		= Name, 
		submodules.Description 	= Description,
		submodules.Type 		= ControlType
	WHERE submodules.ID = SubmoduleID;
	RETURN 0;
END //

DROP FUNCTION IF EXISTS ChangeSubmoduleControlType; //
CREATE FUNCTION `ChangeSubmoduleControlType`(	`TeacherID` 	INT, 
												`SubmoduleID` 	INT, 
												`ControlType`	VARCHAR(30) 
											) RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT submodules.ID 
	INTO checker 
	FROM `submodules`
	INNER JOIN `modules` 		ON 	submodules.ModuleID 	= modules.ID
	INNER JOIN `disciplines` 	ON 	disciplines.ID 			= modules.DisciplineID
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID AND disciplines_teachers.TeacherID 	= TeacherID
	WHERE 	submodules.ID = SubmoduleID 
	LIMIT 1;

	IF checker IS NULL OR checker <= 0 THEN
		RETURN -1;
	END IF;

	UPDATE `submodules` 
	SET submodules.Type 		= ControlType
	WHERE submodules.ID = SubmoduleID;
	RETURN 0;
END //

DROP FUNCTION IF EXISTS ChangeSubmoduleMaxRate; //
CREATE FUNCTION `ChangeSubmoduleMaxRate`(	`TeacherID` 	INT, 
											`SubmoduleID` 	INT, 
											`MaxRate` 		INT

										) 	RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT submodules.ID 
	INTO checker 
	FROM `submodules`
	INNER JOIN `modules` 		ON 	submodules.ModuleID 	= modules.ID
	INNER JOIN `disciplines` 	ON 	disciplines.ID 			= modules.DisciplineID
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID AND disciplines_teachers.TeacherID 	= TeacherID
	WHERE 	submodules.ID = SubmoduleID 
	LIMIT 1;
	IF checker IS NULL OR checker <= 0 THEN
		RETURN -1;
	END IF;

	UPDATE `submodules` 
	SET submodules.MaxRate 		= MaxRate
	WHERE submodules.ID = SubmoduleID;
	RETURN 0;
END //

DROP FUNCTION IF EXISTS ChangeSubmoduleOrder; //
CREATE FUNCTION `ChangeSubmoduleOrder`(	`TeacherID` 	INT, 
										`SubmoduleID` 	INT, 
										`OrderNum` 		INT

									)	RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT modules.ID 
	INTO checker 
	FROM `submodules`
	INNER JOIN `modules` 		ON 	submodules.ModuleID 	= modules.ID
	INNER JOIN `disciplines` 	ON 	disciplines.ID 			= modules.DisciplineID
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID AND disciplines_teachers.TeacherID 	= TeacherID
	WHERE 	submodules.ID = SubmoduleID 
	LIMIT 1;
	IF checker IS NULL OR checker <= 0 THEN
		RETURN -1;
	END IF;


	SET checker = -1;
	SELECT submodules.ID
	INTO checker
	FROM `submodules`
	WHERE submodules.ModuleID = checker AND submodules.OrderNum = OrderNum
	LIMIT 1;
	IF checker > 0 THEN
		RETURN -1;
	END IF; 


	UPDATE `submodules` 
	SET submodules.OrderNum 	= OrderNum
	WHERE submodules.ID = SubmoduleID;
	RETURN 0;
END //

DROP FUNCTION IF EXISTS ChangeSubmoduleName; //
CREATE FUNCTION `ChangeSubmoduleName`(	`TeacherID` 	INT, 
										`SubmoduleID` 	INT, 
										`Name` 			VARCHAR(50) 
								)	RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT submodules.ID 
	INTO checker 
	FROM `submodules`
	INNER JOIN `modules` 		ON 	submodules.ModuleID 	= modules.ID
	INNER JOIN `disciplines` 	ON 	disciplines.ID 			= modules.DisciplineID
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID AND disciplines_teachers.TeacherID 	= TeacherID
	WHERE 	submodules.ID = SubmoduleID 
	LIMIT 1;

	IF checker IS NULL OR checker <= 0 THEN
		RETURN -1;
	END IF;

	UPDATE `submodules` 
	SET 	submodules.Name 		= Name
	WHERE submodules.ID = SubmoduleID;
	RETURN 0;
END //

DROP FUNCTION IF EXISTS ChangeSubmoduleDescription; //
CREATE FUNCTION `ChangeSubmoduleDescription`(	`TeacherID` 	INT, 
												`SubmoduleID` 	INT, 
												`Description` 	VARCHAR(200) 
											) 	RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT submodules.ID 
	INTO checker 
	FROM `submodules`
	INNER JOIN `modules` 		ON 	submodules.ModuleID 	= modules.ID
	INNER JOIN `disciplines` 	ON 	disciplines.ID 			= modules.DisciplineID
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID AND disciplines_teachers.TeacherID 	= TeacherID
	WHERE 	submodules.ID = SubmoduleID 
	LIMIT 1;

	IF checker IS NULL OR checker <= 0 THEN
		RETURN -1;
	END IF;

	UPDATE `submodules` 
	SET submodules.Description 	= Description
	WHERE submodules.ID = SubmoduleID;
	RETURN 0;
END //


-- Внесение изменений в модуль (Title, Order и т.д.), который неявно прикреплен к преподавателю
DROP FUNCTION IF EXISTS ChangeModule; //
CREATE FUNCTION `ChangeModule`(	`TeacherID` INT, 
								`ModuleID` INT, 
								`OrderNum` INT, 
								`Name` VARCHAR(50) ) RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT disciplines.ID 
	INTO checker 
	FROM `modules`
	INNER JOIN `disciplines` ON disciplines.ID = modules.DisciplineID 
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID  AND disciplines_teachers.TeacherID 	= TeacherID
	WHERE modules.ID = ModuleID
	LIMIT 1;
	IF checker IS NULL OR checker <= 0 THEN
		RETURN -1;
	END IF;

	SET checker = -1;
	SELECT modules.ID 
	INTO checker
	FROM `modules`
	WHERE modules.DisciplineID = checker AND modules.OrderNum = OrderNum;
	IF checker > 0 THEN
		RETURN -1;
	END IF;

	UPDATE `modules` 
	SET modules.OrderNum 	= OrderNum, 
		modules.Name 		= Name
	WHERE modules.ID = ModuleID;
	RETURN 0;
END //


DROP FUNCTION IF EXISTS ChangeModuleName; //
CREATE FUNCTION `ChangeModuleName`(	`TeacherID` INT, 
									`ModuleID` INT, 
									`Name` VARCHAR(50) 
								) RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT modules.ID 
	INTO checker 
	FROM `modules`
	INNER JOIN `disciplines` ON disciplines.ID = modules.DisciplineID 
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID  AND disciplines_teachers.TeacherID 	= TeacherID
	WHERE modules.ID = ModuleID
	LIMIT 1;

	IF checker IS NULL OR checker <= 0 THEN
		RETURN -1;
	END IF;

	UPDATE `modules` 
	SET modules.Name 		= Name
	WHERE modules.ID = ModuleID;
	RETURN 0;
END //

DROP FUNCTION IF EXISTS ChangeModuleOrder; //
CREATE FUNCTION `ChangeModuleOrder`	(	`TeacherID` INT, 
										`ModuleID` INT, 
										`OrderNum` INT 
									) RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT disciplines.ID 
	INTO checker 
	FROM `modules`
	INNER JOIN `disciplines` ON disciplines.ID = modules.DisciplineID 
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID  AND disciplines_teachers.TeacherID 	= TeacherID
	WHERE modules.ID = ModuleID
	LIMIT 1;
	IF checker IS NULL OR checker <= 0 THEN
		RETURN -1;
	END IF;

	SET checker = -1;
	SELECT modules.ID 
	INTO checker
	FROM `modules`
	WHERE modules.DisciplineID = checker AND modules.OrderNum = OrderNum;
	IF checker > 0 THEN
		RETURN -1;
	END IF;

	UPDATE `modules` 
	SET modules.OrderNum 	= OrderNum
	WHERE modules.ID = ModuleID;
	RETURN 0;
END //

-- Добавление нового модуля
DROP FUNCTION IF EXISTS AddModule; //
CREATE FUNCTION `AddModule`(	`TeacherID` 	INT, 
								`DisciplineID` 	INT, 
								`OrderNum` 		INT, 
								`Name` 			VARCHAR(50) ) 
								RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT disciplines.ID 
	INTO checker 
	FROM `disciplines`
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.TeacherID = TeacherID AND disciplines_teachers.DisciplineID = disciplines.ID
	WHERE 	disciplines.ID = DisciplineID
	LIMIT 1;
	IF checker IS NULL OR checker <= 0 THEN
		RETURN -1;
	END IF;

	SET checker = -1;
	SELECT modules.ID 
	INTO checker
	FROM `modules`
	WHERE modules.DisciplineID = DisciplineID AND modules.OrderNum = OrderNum;
	IF checker > 0 THEN
		RETURN -2;
	END IF;


	INSERT INTO `modules` 	( modules.Name, modules.OrderNum, modules.DisciplineID )
	VALUES 					( Name , OrderNum , DisciplineID );
	

	RETURN 	(	SELECT 	modules.ID
				FROM `modules`
				WHERE 	modules.DisciplineID = DisciplineID AND 
						modules.OrderNum = OrderNum
			);


END //


-- Добавление нового подмодуля к данному модулю
DROP FUNCTION IF EXISTS AddSubmodule; //
CREATE FUNCTION `AddSubmodule`(	`TeacherID` 	INT, 
								`ModuleID` 		INT, 
								`MaxRate` 		INT, 
								`OrderNum` 		INT, 
								`Name` 			VARCHAR(50), 
								`Description`	VARCHAR(200),
								`ControlType`	VARCHAR(30)

							) RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT modules.ID 
	INTO checker 
	FROM `modules`
	INNER JOIN `disciplines` ON disciplines.ID = modules.DisciplineID
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.TeacherID = TeacherID AND disciplines_teachers.DisciplineID = disciplines.ID
	WHERE 	modules.ID = ModuleID
	LIMIT 1;
	IF checker IS NULL OR checker <= 0 THEN
		RETURN -1;
	END IF;

	SET checker = -1;
	SELECT submodules.ID
	INTO checker
	FROM `submodules`
	WHERE submodules.ModuleID = ModuleID AND submodules.OrderNum = OrderNum
	LIMIT 1;
	IF checker > 0 THEN
		RETURN -1;
	END IF; 

	INSERT INTO `submodules` 	( submodules.ModuleID, submodules.MaxRate, submodules.OrderNum, submodules.Name, submodules.Description, submodules.Type )
	VALUES						( ModuleID, MaxRate, OrderNum, Name, Description, ControlType);
	
	RETURN 	(	SELECT submodules.ID
				FROM `submodules`
				WHERE 	submodules.ModuleID = ModuleID AND
						submodules.OrderNum = OrderNum
			);
END //





-- Добавление нового подмодуля к данному модулю
DROP FUNCTION IF EXISTS DeleteModule; //
CREATE FUNCTION `DeleteModule`(	`TeacherID` 	INT,
								`ModuleID` 	INT 
							)  	RETURNS int(11)
    NO SQL
BEGIN  

    DECLARE checker INT;

	SELECT disciplines.ID 
	INTO checker 
	FROM `modules`
	INNER JOIN `disciplines` ON disciplines.ID = modules.DisciplineID
	INNER JOIN `disciplines_teachers` ON 	disciplines_teachers.TeacherID = TeacherID AND 
											disciplines_teachers.DisciplineID = disciplines.ID
	WHERE 	modules.ID = ModuleID
	LIMIT 1;
	IF checker IS NULL OR checker <= 0 THEN
		RETURN -1;
	END IF;


	SET checker = -1;
	SELECT submodules.ID
	INTO checker
	FROM `submodules`
	WHERE submodules.ModuleID = ModuleID
	LIMIT 1;
	IF checker > 0 THEN
		RETURN -1;
	END IF;


	DELETE FROM `modules` 
	WHERE modules.ID = ModuleID; 


	RETURN 0;
END //


-- Добавление нового подмодуля к данному модулю
DROP FUNCTION IF EXISTS DeleteSubmodule; //
CREATE FUNCTION `DeleteSubmodule`(	`TeacherID` 	INT,
									`SubmoduleID` 	INT
								)  	RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;

	SELECT modules.ID 
	INTO checker 
	FROM `submodules`
	INNER JOIN `modules` ON modules.ID = submodules.ModuleID
	INNER JOIN `disciplines` ON disciplines.ID = modules.DisciplineID
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.TeacherID = TeacherID AND disciplines_teachers.DisciplineID = disciplines.ID
	WHERE 	submodules.ID = SubmoduleID
	LIMIT 1;
	IF checker IS NULL OR checker <= 0 THEN
		RETURN -1;
	END IF;



	SET checker = -1;
	SELECT rating_table.ID
	INTO checker
	FROM `rating_table`
	WHERE rating_table.SubmoduleID = SubmoduleID
	LIMIT 1;
	IF checker > 0 THEN
		RETURN -1;
	END IF;



	DELETE FROM `submodules` 
	WHERE submodules.ID = SubmoduleID; 

	RETURN 0;
END //





DROP FUNCTION IF EXISTS SwapSubmoduleOrder; //
CREATE FUNCTION `SwapSubmoduleOrder`(	`TeacherID` 	INT, 
										`SubmoduleID1` 	INT, 
										`SubmoduleID2` 	INT

									)	RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker, ord1, ord2 INT;  					 
	
	SELECT COUNT(submodules.ID) 
	INTO checker 
	FROM `submodules`
	INNER JOIN `modules` 		ON 	submodules.ModuleID 	= modules.ID
	INNER JOIN `disciplines` 	ON 	disciplines.ID 			= modules.DisciplineID
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID AND disciplines_teachers.TeacherID 	= TeacherID
	WHERE 	submodules.ID = SubmoduleID1 OR submodules.ID = SubmoduleID2;
	IF checker != 2 THEN
		RETURN -1;
	END IF;


	SELECT submodules.OrderNum
	INTO ord1
	FROM `submodules`
	WHERE submodules.ID = SubmoduleID1
	LIMIT 1;

	SELECT submodules.OrderNum
	INTO ord2
	FROM `submodules`
	WHERE submodules.ID = SubmoduleID2
	LIMIT 1;

	UPDATE `submodules` 
	SET submodules.OrderNum 	= 13666
	WHERE submodules.ID = SubmoduleID1;

	UPDATE `submodules` 
	SET submodules.OrderNum 	= ord1
	WHERE submodules.ID = SubmoduleID2;

	UPDATE `submodules` 
	SET submodules.OrderNum 	= ord2
	WHERE submodules.ID = SubmoduleID1;
	RETURN 0;
END //



DROP FUNCTION IF EXISTS SwapModuleOrder; //
CREATE FUNCTION `SwapModuleOrder`	(	`TeacherID` INT, 
										`ModuleID1` INT, 
										`ModuleID2` INT 
									) RETURNS int(11)
    NO SQL
BEGIN  
   	DECLARE checker, ord1, ord2 INT; 				 
	
	SELECT COUNT(modules.ID) 
	INTO checker 
	FROM `modules`
	INNER JOIN `disciplines` ON disciplines.ID = modules.DisciplineID 
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID  AND disciplines_teachers.TeacherID 	= TeacherID
	WHERE modules.ID = ModuleID1 OR modules.ID = ModuleID2
	LIMIT 2;
	IF checker != 2 THEN
		RETURN -1;
	END IF;

	SELECT modules.OrderNum
	INTO ord1
	FROM `modules`
	WHERE modules.ID = ModuleID1
	LIMIT 1;

	SELECT modules.OrderNum
	INTO ord2
	FROM `modules`
	WHERE modules.ID = ModuleID2
	LIMIT 1;

	UPDATE `modules` 
	SET modules.OrderNum 	= 13666
	WHERE modules.ID = ModuleID1;

	UPDATE `modules` 
	SET modules.OrderNum 	= ord1
	WHERE modules.ID = ModuleID2;

	UPDATE `modules` 
	SET modules.OrderNum 	= ord2
	WHERE modules.ID = ModuleID1;
	
	RETURN 0;
END //


-- ====== Requests =======  --


DROP PROCEDURE IF EXISTS GetLastRequests; //
CREATE PROCEDURE `GetLastRequests`( IN `CountN` INT )
    NO SQL
BEGIN 
	DECLARE offsetN, allNum, countClone INT;
	SELECT COUNT(requests.ID) INTO allNum FROM `requests`;
	SET countClone = CountN;
	IF CountN = -1 THEN 
		SET countClone = allNum;
	END IF;

	SET offsetN = allNum - CountN;
	IF offsetN < 0 THEN
		SET offsetN = 0;
	END IF;

	SELECT 	requests.ID,
			requests.AccountID,
			requests.Information,
			requests.Date,
			requests.Status
	FROM `requests`
	WHERE requests.Status = 0
	LIMIT countClone OFFSET offsetN;

END //


DROP PROCEDURE IF EXISTS GetFirstRequests ; //
CREATE PROCEDURE `GetFirstRequests`( IN `CountN` INT )
    NO SQL
BEGIN 
	DECLARE allNum, countClone INT;
	SELECT COUNT(requests.ID) INTO allNum FROM `requests`;
	SET countClone = CountN;
	IF CountN = -1 THEN 
		SET countClone = allNum;
	END IF;

	SELECT 	requests.ID,
			requests.AccountID,
			requests.Information,
			requests.Date,
			requests.Status
	FROM `requests`
	WHERE requests.Status = 0
	LIMIT countClone;

END //




